@@ -4,8 +4,10 @@ from pathlib import Path
 from typing import Union, List, Callable, Optional
 from dataclasses import dataclass
 
-from .._numpy_utils import as_array, has_floats
-from ..predicates import FuzzyEquality, ExactEquality
+from ..predicates import DefaultEquality
+from ..protocols import DynamicTolerance
+from ..io import read, read_as
+
 from .._common import _default_base_tolerance
 from .._format import as_success, as_error, as_warning, highlighted
 
@@ -19,12 +21,14 @@ from .._field_data_comparison import (
 from ..mesh import MeshFieldsComparator
 
 from ._logger import CLILogger
+from ._common import FileTypeMap
 from ._test_suite import TestSuite, TestResult, TestStatus
 
 from .. import protocols
 from ..mesh import protocols as mesh_protocols
 
-from ..io import read as read_fields
+
+Tolerance = Union[float, DynamicTolerance]
 
 
 @dataclass
@@ -33,13 +37,14 @@ class FileComparisonOptions:
     ignore_missing_reference_fields: bool = False
     ignore_missing_sequence_steps: bool = False
     force_sequence_comparison: bool = False
-    relative_tolerances: Callable[[str], float] = lambda _: _default_base_tolerance()
-    absolute_tolerances: Callable[[str], float] = lambda _: _default_base_tolerance()
+    relative_tolerances: Callable[[str], Optional[Tolerance]] = lambda _: _default_base_tolerance()
+    absolute_tolerances: Callable[[str], Optional[Tolerance]] = lambda _: 0.0
     field_inclusion_filter: Callable[[str], bool] = lambda _: True
     field_exclusion_filter: Callable[[str], bool] = lambda _: False
     disable_unconnected_points_removal: bool = False
     disable_mesh_space_dimension_matching: bool = False
     disable_mesh_reordering: bool = False
+    file_type_map: FileTypeMap = FileTypeMap()
 
 
 class FileComparison:
@@ -58,11 +63,17 @@ class FileComparison:
         return self._compare_fields(res_fields, ref_fields).with_overridden(name=_suite_name(res_file))
 
     def _read(self, filename: str) -> Union[protocols.FieldData, protocols.FieldDataSequence]:
-        self._logger.log(f"Reading '{highlighted(filename)}'\n", verbosity_level=1)
         try:
-            return read_fields(filename)
+            file_type_with_opts = self._opts.file_type_map(filename)
+            log_suffix = f" as '{file_type_with_opts[0]}'" if file_type_with_opts is not None else ""
+            self._logger.log(f"Reading '{highlighted(filename)}'{log_suffix}\n", verbosity_level=1)
+            return (
+                read(filename)
+                if file_type_with_opts is None
+                else read_as(file_type_with_opts[0], filename, **file_type_with_opts[1])
+            )
         except IOError as e:
-            self._logger.log(f"Error: '{e}'", verbosity_level=1)
+            self._logger.log(f"Error: '{e}'\n", verbosity_level=1)
             raise IOError(e)
 
     def _compare_fields(
@@ -92,9 +103,9 @@ class FileComparison:
         if res_sequence.number_of_steps != ref_sequence.number_of_steps:
             if not self._opts.ignore_missing_sequence_steps:
                 num_steps_check = TestStatus.failed
-                self._logger.log(f"{self._status_string(TestStatus.error)}: {num_steps_check_fail_msg}")
+                self._logger.log(f"{num_steps_check_fail_msg}\n")
                 if not self._opts.force_sequence_comparison:
-                    return _make_test_suite([], TestStatus.failed, num_steps_check_fail_msg)
+                    return _make_test_suite([], TestStatus.failed, shortlog=num_steps_check_fail_msg)
             else:
                 self._logger.log(f"{as_warning('Warning')}: {num_steps_check_fail_msg}, comparing only common steps\n")
 
@@ -125,7 +136,12 @@ class FileComparison:
     def _compare_field_data(self, res_fields: protocols.FieldData, ref_fields: protocols.FieldData) -> TestSuite:
         if isinstance(res_fields, mesh_protocols.MeshFields) and isinstance(ref_fields, mesh_protocols.MeshFields):
             return self._compare_mesh_field_data(res_fields, ref_fields)
-        return self._to_test_suite(self._run_field_data_comparison(res_fields, ref_fields))
+        suite = self._run_field_data_comparison(res_fields, ref_fields)
+        if suite.domain_equality_check:
+            return self._to_test_suite(suite)
+        msg = "Domains have compared unequal"
+        self._logger.log(f"{msg}\n")
+        return _make_test_suite([], TestStatus.failed, shortlog=msg)
 
     def _compare_mesh_field_data(
         self, res_fields: mesh_protocols.MeshFields, ref_fields: mesh_protocols.MeshFields
@@ -137,7 +153,7 @@ class FileComparison:
             if suite.domain_equality_check:
                 return self._to_test_suite(suite)
             msg = "Non-reordered meshes have compared unequal"
-            self._logger.log(f"{self._status_string(TestStatus.failed)}: {msg}")
+            self._logger.log(f"{msg}\n")
             return _make_test_suite([], TestStatus.failed, shortlog=msg)
 
         suite = self._run_mesh_fields_comparison(res_fields, ref_fields)
@@ -145,8 +161,8 @@ class FileComparison:
             return self._to_test_suite(suite)
 
         msg = "Fields defined on different meshes"
-        self._logger.log(f"{self._status_string(TestStatus.failed)}: {msg}")
-        return _make_test_suite([], TestStatus.failed, shortlog="Fields defined on different meshes")
+        self._logger.log(f"{msg}\n")
+        return _make_test_suite([], TestStatus.failed, shortlog=msg)
 
     def _run_mesh_fields_comparison(
         self, result: mesh_protocols.MeshFields, reference: mesh_protocols.MeshFields
@@ -196,13 +212,12 @@ class FileComparison:
         )
 
     def _select_predicate(self, res_field: protocols.Field, ref_field: protocols.Field) -> protocols.Predicate:
-        if has_floats(as_array(res_field.values)) or has_floats(as_array(ref_field.values)):
-            return FuzzyEquality(
-                abs_tol=self._opts.absolute_tolerances(res_field.name),
-                rel_tol=self._opts.relative_tolerances(res_field.name),
-            )
-        else:
-            return ExactEquality()
+        abs_tol = self._opts.absolute_tolerances(res_field.name)
+        rel_tol = self._opts.relative_tolerances(res_field.name)
+        return DefaultEquality(
+            abs_tol=abs_tol if abs_tol is not None else 0.0,
+            rel_tol=rel_tol if rel_tol is not None else _default_base_tolerance(),
+        )
 
     def _to_test_suite(self, suite: FieldComparisonSuite) -> TestSuite:
         return TestSuite([self._to_test_result(c) for c in suite])
