@@ -1,11 +1,11 @@
 """Class to represent computational meshes"""
 
 from __future__ import annotations
-from typing import Iterable, Tuple, Optional
+from typing import Iterable, Tuple, Optional, Union
 
-from .._common import _default_base_tolerance
-from .._numpy_utils import Array, ArrayLike, as_array
 from ..predicates import PredicateResult
+from ..protocols import DynamicTolerance
+from .._numpy_utils import Array, ArrayLike, as_array, max_abs_value
 
 from ._mesh_equal import mesh_equal
 from ._cell_type import CellType
@@ -13,6 +13,11 @@ from ._cell_type import CellType
 from . import protocols
 
 
+def default_mesh_relative_tolerance() -> float:
+    """Return the default relative tolerance used for mesh equality checks"""
+    return 1e-8
+
+
 class Mesh:
     """
     Represents a computational mesh.
@@ -25,8 +30,8 @@ class Mesh:
     def __init__(self, points: ArrayLike, connectivity: Iterable[Tuple[CellType, ArrayLike]]) -> None:
         self._points = as_array(points)
         self._corners = {_get_assert_cell_type(cell_type): as_array(corners) for cell_type, corners in connectivity}
-        self._abs_tol = _default_base_tolerance()
-        self._rel_tol = _default_base_tolerance()
+        self._rel_tol = default_mesh_relative_tolerance()
+        self._abs_tol = max_abs_value(self._points) * default_mesh_relative_tolerance()
 
     @property
     def absolute_tolerance(self) -> float:
@@ -64,9 +69,13 @@ class Mesh:
         Args:
             other: mesh against with to check equality.
         """
-        return mesh_equal(self, other, abs_tol=self._abs_tol, rel_tol=self._rel_tol)
+        return mesh_equal(self, other)
 
-    def set_tolerances(self, abs_tol: Optional[float] = None, rel_tol: Optional[float] = None) -> None:
+    def set_tolerances(
+        self,
+        abs_tol: Optional[Union[float, DynamicTolerance]] = None,
+        rel_tol: Optional[Union[float, DynamicTolerance]] = None,
+    ) -> None:
         """
         Set the tolerances to be used for equality checks against other meshes.
 
@@ -74,8 +83,13 @@ class Mesh:
             abs_tol: Absolute tolerance to use.
             rel_tol: Relative tolerance to use.
         """
-        self._abs_tol = abs_tol if abs_tol is not None else self._abs_tol
-        self._rel_tol = rel_tol if rel_tol is not None else self._rel_tol
+        self._rel_tol = self._rel_tol if rel_tol is None else self._get_tolerance(rel_tol)
+        self._abs_tol = self._abs_tol if abs_tol is None else self._get_tolerance(abs_tol)
+
+    def _get_tolerance(self, tol: Union[float, DynamicTolerance]) -> float:
+        result = tol(self._points, self._points) if isinstance(tol, DynamicTolerance) else tol
+        assert isinstance(result, float)
+        return result
 
 
 def _get_assert_cell_type(cell_type: CellType) -> CellType:
