@@ -2,8 +2,4 @@
 
 from ._tabular_fields import Table, TabularFields, transform
 
-__all__ = [
-    "Table",
-    "TabularFields",
-    "transform"
-]
+__all__ = ["Table", "TabularFields", "transform"]
